
/**
 * Muestra un archivo en hex
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */

import java.io.*;

public class HexDump
{

    public static void main(String[] args) {
        String filename;
        // vemos si hay algún file especificado en la linea de comando
        if (args.length != 0) {
            filename = args[0];
        } else {
            filename = "HexDump.class";
        }
        
        try {
            HexDump.readFile(filename);
        } catch (IOException e) {
            System.out.println(e.getMessage());
            System.out.println("El programa terminó.");
        }
        
    }
    
    /**
     * Lee el archivo filename y lo imprime en Sytem.out en dos columnas:
     * la de la izquierda muestra el archivo en bytes, la de la 
     * derecha muestra los caracteres imprimibles.
     * @param filename  el archivo que se imprimirá en pantalla
     * @throws IOException si a) el archivo no existe o b) hay un error de lectura
     */
    public static void readFile(String filename) throws IOException {
    
        final int lineLength = 16;           // leemos de 8 en 8
        int[] buffer = new int[lineLength];   // un arreglo para leer el archivo
        int c;
        int iCounter = 0;

        File file = new File(filename);
        FileInputStream in;
        
        try {
            // Este constructor arroja una excepción si el archivo no existe o no se puede leer.
            in = new FileInputStream( file );
        } catch (IOException e) {
            // la pasamos al método que nos llamó
            throw new IOException("No existe el archivo " + filename + ", o no es un archivo: " + e.getMessage() );
        } 
        
        
        // Imprimimos la ruta completa del archivo
        System.out.println("[" + file.getAbsolutePath() + "]");
        
        // usamos try, porque read() puede arrojar un IOException
        try {
            while ( (c = in.read()) != -1 ) {   // read devuelve -1 si se acabó el archivo
                buffer[iCounter] = c;
                // cada lineLength caracteres, imprimos la línea y reseteamos el contador
                // para reusar el arreglo
                if (iCounter == lineLength -1 ) {
                    prettyPrint(buffer);
                    iCounter = 0;
                } else {
                    iCounter++;
                } // end if
            
            } // end while
    
        } catch (IOException e) {
            // simplemente reelanzamos la excepción
            throw new IOException("Ocurrió un error al leer el archivo: " + e.getMessage() );
        }
        
        in.close( );  // importante, no olvidarnos de cerrar el stream
        
    }
    
    
    private static void prettyPrint( int[] buffer ) {
        // imprimimos la línea de bytes en formato hex de dos dígitos (%02x)
        for( int c : buffer ) {
            System.out.print( String.format("%02x ",  c ) );
        }
        System.out.print("    ");
        
        // imprimimos la línea, pero sólo los caracteres imprimibles
        // Los que no se pueden imprimir los reemplazamos por un punto
        for( int c : buffer ) {
            if ( Character.isLetterOrDigit( c ) ) {
                System.out.print( Character.toChars( c ));
            } else {
                System.out.print(".");
            }
        }
        System.out.println();
    }

}
